@@ -159,8 +159,8 @@ public:
 	void clear();
 
 private:
-	uint64_t number_value_integer() const;
-	double number_value_double() const;
+	uint64_t FZ_PRIVATE_SYMBOL number_value_integer() const;
+	double FZ_PRIVATE_SYMBOL number_value_double() const;
 
 	bool FZ_PRIVATE_SYMBOL check_type(json_type t);
 	void FZ_PRIVATE_SYMBOL set_type(json_type t);
