@@ -1401,6 +1401,52 @@ std::unique_ptr<socket> listen_socket::accept(int &error, fz::event_handler * ha
 	return ret;
 }
 
+std::unique_ptr<listen_socket> listen_socket::from_descriptor(socket_descriptor && desc, thread_pool & pool, int & error, fz::event_handler * handler)
+{
+	if (!desc) {
+		error = ENOTSOCK;
+		return nullptr;
+	}
+
+#ifndef FZ_WINDOWS
+	disable_sigpipe();
+#endif
+
+	socket_t fd = desc.detach();
+
+#if defined(SO_NOSIGPIPE) && !defined(MSG_NOSIGNAL)
+	// We do not want SIGPIPE if writing to socket.
+	const int value = 1;
+	setsockopt(fd, SOL_SOCKET, SO_NOSIGPIPE, &value, sizeof(int));
+#endif
+
+	set_nonblocking(fd);
+
+	auto pSocket = std::make_unique<listen_socket>(pool, nullptr);
+	if (!pSocket->socket_thread_) {
+		error = ENOMEM;
+		close_socket_fd(fd);
+		return nullptr;
+	}
+	pSocket->state_ = listen_socket_state::listening;
+	pSocket->fd_ = fd;
+
+	pSocket->evt_handler_ = handler;
+	pSocket->socket_thread_->waiting_ = WAIT_ACCEPT;
+
+	sockaddr_u addr;
+	socklen_t addr_len = sizeof(addr);
+	if (!getsockname(pSocket->fd_, &addr.sockaddr_, &addr_len)) {
+		pSocket->family_ = addr.sockaddr_.sa_family;
+	}
+	if (pSocket->socket_thread_->start()) {
+		error = ENOMEM;
+		pSocket.reset();
+	}
+
+	return pSocket;
+}
+
 socket_descriptor listen_socket::fast_accept(int &error)
 {
 	if (!socket_thread_) {
